#!/usr/bin/env bash

# set -x # turn on for debug

set -o nounset
set -o errexit

script_dir=$(cd "$(dirname "${0}")" && pwd)
cd "${script_dir}" || exit 1

function rand_string() {
  echo "${RANDOM}" | shasum -a 256 | head -c 20
}

function download_software_tar() {
    local _url="${1}" # what to download
    local _output_name="${2}" # where to store extracted contents of archive

    local _tmp_output
    _tmp_output="$(rand_string).tar.gz"

    printf "\t- downloading '%s'...\n" "${_output_name}"

    curl --location \
         --fail \
         --output "${_tmp_output}" \
         "${_url}"

    mkdir -p "${_output_name}"

    tar xf \
        "${_tmp_output}" \
        --directory="./${_output_name}"
    rm -f "${_tmp_output}"
}

function os_name() {
    if [ "$(uname -s)" == "Darwin" ]; then
        echo "macos"
        return
    fi

    if [ -f /etc/os-release ] &&
           grep -q -i 'alpine' /etc/os-release; then
        echo "alpine"
        return
    fi

    echo "linux"
}

function install_rocksdb_ci() {
    local _os
    _os="$(os_name)"

    local _arch
    local _target_arch

    if [ ! -z "${GO_TARGET_ARCH:-}" ];then
        _target_arch="${GO_TARGET_ARCH}"
    else
        _target_arch="$(uname -m)"
    fi

    case "${_target_arch}" in
        "arm64"|"aarch64") _arch="arm64" ;;
        "amd64"|"x86_64"|"x86") _arch="amd64" ;;
        *)
            echo "unknown arch: $_target_arch"
            exit 1
            ;;
    esac

    local _rocksdb_release_static="rocksdb-static-lib-8.3.2-${_os}-${_arch}"
    local _rocksdb_release_shared="rocksdb-shared-lib-8.3.2-${_os}-${_arch}"
    local _rocksdb_release_combined="rocksdb-combined-lib-8.3.2-${_os}-${_arch}"


    echo "Downloading RocksDB build, OS=${_os}, ARCH=${_arch}..."

    if [ -e "${_rocksdb_release_combined}" ]; then
        printf "\t- '%s' already is installed, setting symlink just in case.\n" "${_rocksdb_release_combined}"
        ln -sf "${_rocksdb_release_combined}" "rocksdb-ci-libs"
        return
    fi

    download_software_tar \
        "https://cdn.mixcr.com/internal/pkgs/rocksdb/8.3.2/${_rocksdb_release_static}.tar.gz" \
        "${_rocksdb_release_combined}"

    download_software_tar \
        "https://cdn.mixcr.com/internal/pkgs/rocksdb/8.3.2/${_rocksdb_release_shared}.tar.gz" \
        "${_rocksdb_release_combined}"

    ln -sf "${_rocksdb_release_combined}" "rocksdb-ci-libs"

}

function install_rocksdb() {
    local _os
    _os="$(os_name)"

    local _arch
    local _target_arch

    if [ ! -z "${GO_TARGET_ARCH:-}" ];then
        _target_arch="${GO_TARGET_ARCH}"
    else
        _target_arch="$(uname -m)"
    fi

    case "${_target_arch}" in
        "arm64"|"aarch64") _arch="arm64" ;;
        "amd64"|"x86_64"|"x86") _arch="amd64" ;;
        *)
            echo "unknown arch: $_target_arch"
            exit 1
            ;;
    esac

    local _rocksdb_release="rocksdb-static-lib-8.3.2-${_os}-${_arch}"

    echo "Downloading RocksDB build, OS=${_os}, ARCH=${_arch}..."

    if [ -e "${_rocksdb_release}" ]; then
        printf "\t- '%s' already is installed, setting symlink just in case.\n" "${_rocksdb_release}"
        ln -sf "${_rocksdb_release}" "rocksdb"
        return
    fi

    download_software_tar \
        "https://cdn.mixcr.com/internal/pkgs/rocksdb/8.3.2/${_rocksdb_release}.tar.gz" \
        "${_rocksdb_release}"

    ln -sf "${_rocksdb_release}" "rocksdb"
    echo ""
}

if [ ! -z "${CI:-}" ];then
    printf "\t- Installing RocksDB for CI.\n"
    install_rocksdb_ci
else
    install_rocksdb
fi

echo "Done."
